<?php
/** 
*
* @package phpBB3
* @version $Id: smartfeed.php, v 2.0.0 Alpha Version 1 2008/01/08 20:49:00 mhamill@computer.org Exp $
* @copyright (c) Mark D. Hamill (mhamill@computer.org, http://phpbb.potomactavern.org) 
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* @ignore
*/

// Written by Mark D. Hamill, mhamill@computer.org, http://phpbb.potomactavern.org
// This software is designed to work with phpBB Version 3.0 Gold

// This is the program that creates the RSS or Atom compliant newsfeed that will also include protected
// forums. An enhanced and slightly modified version of the the FeedCreator class, v1.8.0-dev was used 
// to generate the various feeds. The modified version can be found at 
// http://blog.mypapit.net/2005/11/using-feedcreator-to-generate-atom-10-feeds.html.

// A companion program should be run first: smartfeed_url.php. This creates a URL that is
// used to invoke the feed. Use the URL created by this program in your newsreader. Do NOT try to run 
// this program without arguments!

// Set up the phpBB environment we need 
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// We need to start a session to take advantage of the benefits it gives, like access to common function, sessions, etc.
$user->session_begin();
$auth->acl($user->data);
$user->setup();

// If the board is currently disabled, feeds should also be disabled, don't ya think?
if ($config['board_disable'])
{
	handle_error(SMARTFEED_BOARD_DISABLED);
}

// Set up some variables
$registered_user = false;
$allowed_user_types = array(USER_NORMAL, USER_FOUNDER); // Allowed users are Normal and Founder. Others (Inactive, Ignore) can only get a public feed.
$board_url = generate_board_url() . '/';

// ------ First, report errors for any inconsistent URL errors ------ //

// Arguments are required when running smartfeed.php
if (strlen($_SERVER['QUERY_STRING']) == 0)
{
	handle_error(SMARTFEED_NO_ARGUMENTS);
}

// Get the user id. A public user won't be identified as a user in the URL.
$user_id = request_var('u', -1);

// Get the encrypted password. When decrypted it is still encoded md5 as it should also be in the database
$encrypted_pswd = request_var('e', '-1');

// Determine if this is a public request. If so only public forums will be shown.
if ($user_id <> -1 && $encrypted_pswd <> '-1')
{
	// Feed attributes are dependent upon the auth_method. This code makes this program consistent with smartfeed_url.php
	switch ($config['auth_method'])
	{
		case 'db':
			$registered_user = true;
			break;
		case 'apache':
			if (SMARTFEED_APACHE_HTACCESS_ENABLED)
			{
				$registered_user = true;
			}
			else
			{
				handle_error(SMARTFEED_APACHE_AUTHENTICATION_WARNING_REG);
			}
			break;
		case 'ldap':
			$user_id = ANONYMOUS;
			break;
		default:
			// Should never occur
			handle_error(SMARTFEED_BAD_AUTH_METHOD);
			break;
	}
}
else if (($user_id == -1) && ($encrypted_pswd == '-1'))
{
	// Public User
	$user_id = ANONYMOUS;
}
else
{
	// Logically if only the u or the e parameter is present, the URL is inconsisent, so generate an error.
	if ($user_id == -1)
	{
		handle_error(SMARTFEED_NO_U_ARGUMENT);
	}
	if ($encrypted_pswd == -1)
	{
		handle_error(SMARTFEED_NO_E_ARGUMENT);
	}
}

// Get the limit parameter. It limits the size of the newsfeed to a point in time from the present, either a day/hour/minute interval, 
// or the time since the user's last visit. It should always exist. If it doesn't exist or the value passed is invalid, trigger an error.
// Please note that no message older than SMARTFEED_DEFAULT_FETCH_TIME_LIMIT can actually be retrieved to avoid huge newsfeeds and 
// excessive strain on the database.
$time_limit = request_var('limit','');
if ($registered_user)
{
	if ($time_limit == '' ||
		!(($time_limit == $user->lang['SMARTFEED_SINCE_LAST_VISIT']) ||
		($time_limit == $user->lang['SMARTFEED_LAST_QUARTER_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_MONTH_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_TWO_WEEKS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_WEEK_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_DAY_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_12_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_6_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_3_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_1_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_30_MINUTES_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_15_MINUTES_VALUE'])))
	{
		handle_error(SMARTFEED_LIMIT_FORMAT_ERROR);
	}
}
else
{
	if ($time_limit == '' ||
		!(($time_limit == $user->lang['SMARTFEED_LAST_QUARTER_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_MONTH_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_TWO_WEEKS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_WEEK_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_DAY_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_12_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_6_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_3_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_1_HOURS_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_30_MINUTES_VALUE']) || 
		($time_limit == $user->lang['SMARTFEED_LAST_15_MINUTES_VALUE'])))
	{
		handle_error(SMARTFEED_LIMIT_FORMAT_ERROR);
	}
}

// Validate the feed type. Atom 1.0, RSS 2.0 and RSS 1.0 are the only feed types allowed.
$feed_type = request_var('feed_type','');
if (($feed_type == '') || !(($feed_type == SMARTFEED_ATOM_10_VALUE) || 
	($feed_type == SMARTFEED_RSS_20_VALUE) || 
	($feed_type == SMARTFEED_RSS_10_VALUE)))
{
	// Bad feed_type, trigger error
	handle_error(SMARTFEED_FEED_TYPE_ERROR);
}

// Validate the sort_by parameter, which should always be present
$order_by = request_var('sort_by','');
if (($order_by == '') || !(($order_by == 'user') ||
	($order_by == 'standard') ||
	($order_by == 'standard_desc') ||
	($order_by == 'postdate') ||
	($order_by == 'postdate_desc')))
{
	// Bad sort_bye, trigger error
	handle_error(SMARTFEED_SORT_BY_ERROR);
}

// Validate the new topics only parameter. If it does not show, it is not true. Any value other than 1 is false.
$topics_only = request_var('topicsonly','0');
if (!(($topics_only == '0') || ($topics_only == '1')))
{
	// Bad topics only value, trigger error
	handle_error(SMARTFEED_TOPICS_ONLY_ERROR);
}
$topics_only = intval($topics_only);

// Validate the firstpostonly parameter. It should only appear if the topicsonly parameter is present
if ($topics_only == 1)
{
	$first_post_only = request_var('firstpostonly','0');
	if (!(($first_post_only == '0') || ($first_post_only == '1')))
	{
		// Bad firstpostonly value, trigger error
		handle_error(SMARTFEED_FIRST_POST_ONLY_ERROR);
	}
	$first_post_only = intval($first_post_only);
}
else
{
	$first_post_only = 0;
}

// Check for count limit parameter. It is not required, but if present should be a positive number only. It only has meaning if 
// the sort_by parameter has the value "postdate_desc"
$count_limit = request_var('count_limit','');
if ($count_limit <> '')
{
	if ($order_by <> 'postdate_desc')
	{
		handle_error(SMARTFEED_COUNT_LIMIT_CONSISTENCY_ERROR);
	}
	else if (!( intval($count_limit > 0) ))
	{
		handle_error(SMARTFEED_COUNT_LIMIT_ERROR);
	}
}
$count_limit = intval($count_limit);

// Validate the maximum size requested for the number of words in a post
$max_word_size = request_var('max_word_size','');
if (!(($max_word_size == $user->lang['SMARTFEED_MAX_WORDS_WANTED']) || (intval($max_word_size > 0)) ))
{
	// Bad max_word_size, trigger error
	handle_error(SMARTFEED_MAX_WORD_SIZE_ERROR);
}
	
// Validate the minimum size requested for the number of words in a post
$min_word_size = request_var('min_word_size','');
if ($min_word_size <> '')
{
	$min_word_size = intval($min_word_size);
	if ($min_word_size <= 0) 	// Note: characters in the field will evaluate to 0
	{
		handle_error(SMARTFEED_MIN_WORD_SIZE_ERROR);
	}
}
	
// Validate the feed_style parameter.
$feed_style = request_var('feed_style','');
if (!(($feed_style == 'COMPACT') || ($feed_style == 'BASIC')  || ($feed_style == 'HTML')))
{
	handle_error(SMARTFEED_STYLE_ERROR);
}

if ($registered_user)
{
	// Validate the removemine parameter, if present
	$remove_mine = request_var('removemine','');
	if (!($remove_mine == '' || $remove_mine == '1'))
	{
		// Bad sort_bye, trigger error
		handle_error(SMARTFEED_REMOVE_MINE_ERROR);
	}

	// Validate the private messages switch
	$show_pms = request_var('pms','');
	if (($show_pms <> '1') && ($show_pms <> ''))
	{
		// Bad parameter value for private messages
		handle_error(SMARTFEED_BAD_PMS_VALUE);
	}
	$show_pms = intval($show_pms);

	// Validate the bookmarks switch
	$bookmarks = request_var('bookmarks','');
	if (($bookmarks <> '1') && ($bookmarks <> ''))
	{
		// Bad parameter value for private messages
		handle_error(SMARTFEED_BAD_BOOKMARKS_VALUE);
	}
	$bookmarks = intval($bookmarks);

	// Validate the filter_foes parameter.
	$filter_foes = request_var('filter_foes','');
	if (!(($filter_foes == '') || ($filter_foes == '1')))
	{
		handle_error(SMARTFEED_FILTER_FOES_ERROR);
	}
	$filter_foes = intval($filter_foes);
	
	// Validate the lastvisit parameter.
	$lastvisit = request_var('lastvisit','');
	if (!(($lastvisit == '') || ($lastvisit == '1')))
	{
		handle_error(SMARTFEED_LASTVISIT_ERROR);
	}
	$lastvisit = intval($lastvisit);

}

// ------ Second, authenticate the user ------ //

// We need the user_permissions column even for Anonymous, so we make this database call even if they cannot log in
$sql = 'SELECT user_password, user_permissions, user_smartfeed_key, user_topic_sortby_type, user_topic_sortby_dir, 
			user_post_sortby_type, user_post_sortby_dir, user_lastvisit, user_type
			FROM ' . USERS_TABLE . "
			WHERE user_id = $user_id";

if ( !($result = $db->sql_query($sql)) )
{
	handle_error(SMARTFEED_USER_TABLE_PASSWORD_ERROR);
}

$rowset = $db->sql_fetchrowset($result);

if (count($rowset) == 0)
{
	handle_error(SMARTFEED_USER_ID_DOES_NOT_EXIST);
}

// Make sure user_id exists in database and has normal or founder status
$row = reset($rowset);
$normal_user = (in_array($row['user_type'], $allowed_user_types)) ? true : false;
if ( !$normal_user )
{
	handle_error(SMARTFEED_USER_TABLE_COUNT_ERROR);
}

// Save the user variables, although only the first is unneeded for guests.
$user_permissions = $row['user_permissions'];
$user_smartfeed_key = $row['user_smartfeed_key'];

// These other variables can be used to order posts, although they apply only to registered users
$user_password = $row['user_password'];
$user_topic_sortby_type = $row['user_topic_sortby_type']; 
$user_topic_sortby_dir = $row['user_topic_sortby_dir']; 
$user_post_sortby_type = $row['user_post_sortby_type']; 
$user_post_sortby_dir = $row['user_post_sortby_dir'];
$user_lastvisit = $row['user_lastvisit'];

$db->sql_freeresult($result);

if ($registered_user)
{
	
	// + signs in $HTTP_GET_VARS['e'] seem to get translated to a space character, so put them back in
	$encrypted_pswd = str_replace(' ','+',$encrypted_pswd);
	
	// Decrypt password using the user_smartfeed_key column in the phpbb_users table. This should have gotten created 
	// the first time smartfeed_url.php was run by this user. Note the encoded password is typically md5. There should not be
	// a clear text password in the database.
	if (strlen($user_smartfeed_key) > 0)
	{
		$encoded_pswd = decrypt($encrypted_pswd, $user_smartfeed_key);
	}
	else
	{
		// If the $user_smartfeed_key is null, password cannot be decrypted. This avoids PHP Notice statements in TripleDES.php
		handle_error(SMARTFEED_BAD_PASSWORD_ERROR);
	}
				
	// If IP Authentication was enabled, the encoded password is to the left of the ~ and the IP to the right of the ~
	$tilde = strpos($encoded_pswd, '~');
	if (($tilde == 0) && SMARTFEED_REQUIRE_IP_AUTHENTICATION)
	{
		handle_error(SMARTFEED_IP_AUTH_ERROR);
	}
	
	if ($tilde > 0)
	{
		// Since a tilde is present, authenticate the client IP by comparing it with the IP embedded in the "e" parameter
		$auth_ip = substr($encoded_pswd,$tilde+1);
		$encoded_pswd = substr($encoded_pswd,0,$tilde);
		$client_ip_parts = explode('.', $_SERVER['REMOTE_ADDR']);
		$source_ip_parts = explode('.', $auth_ip);
		// Show error message if requested from incorrect range of IP addresses
		if (!(($client_ip_parts[0] == $source_ip_parts[0]) && ($client_ip_parts[1] == $source_ip_parts[1]) && ($client_ip_parts[2] == $source_ip_parts[2])))
		{
			handle_error(SMARTFEED_IP_AUTH_ERROR);
		}
	}

	if ($encoded_pswd <> $user_password)
	{
		handle_error(SMARTFEED_BAD_PASSWORD_ERROR);
	} 

}

// If requested to fetch new posts since the user's last visit, we need to get the user_lastvisit date from the database. Of course this makes
// no sense if the user is Anonymous. Cookie logic in the phpBB 2 version has been removed to simplify things. Most newsreaders won't store cookies
// anyway.
$date_limit_sql = '';
if ($registered_user && ($time_limit == $user->lang['SMARTFEED_SINCE_LAST_VISIT'])) // i.e. use the user's last visit and show all selected items after this time in the feed
{
	$date_limit = $user_lastvisit;
}
else
{
	switch ($time_limit)
	{
		case $user->lang['SMARTFEED_LAST_QUARTER_VALUE']:
			$date_limit = time() - (90 * 24 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_MONTH_VALUE']:
			$date_limit = time() - (30 * 24 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_TWO_WEEKS_VALUE']:
			$date_limit = time() - (14 * 24 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_WEEK_VALUE']:
			$date_limit = time() - (7 * 24 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_DAY_VALUE']:
			$date_limit = time() - (24 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_12_HOURS_VALUE']:
			$date_limit = time() - (12 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_6_HOURS_VALUE']:
			$date_limit = time() - (6 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_3_HOURS_VALUE']:
			$date_limit = time() - (3 * 60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_1_HOURS_VALUE']:
			$date_limit = time() - (60 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_30_MINUTES_VALUE']:
			$date_limit = time() - (30 * 60);
			break;
		case $user->lang['SMARTFEED_LAST_15_MINUTES_VALUE']:
			$date_limit = time() - (15 * 60);
	}
}

$date_limit_sql = ' AND p.post_time > ' . $date_limit;

// ------ Third, create a list of forums that will be fetched ------ //

$fetched_forum_str = '';

if ($registered_user && $bookmarks)
{

	// When selecting bookmarked topics only, we can safely ignore the logic constraining the user to read only 
	// from certain forums. Instead we will create the SQL to get the bookmarked topics, if any, hijacking the 
	// $fetched_forum_str variable since it is convenient
	
	$bookmarked_topics = array();
	$sql = 'SELECT t.topic_id
				FROM ' . USERS_TABLE . ' u, ' . BOOKMARKS_TABLE . ' b, ' . TOPICS_TABLE . " t
				WHERE u.user_id = b.user_id AND b.topic_id = t.topic_id
					AND t.topic_last_post_time > $date_limit
					AND b.user_id = $user_id";
	if ( !($result = $db->sql_query($sql)))
	{
		handle_error(SMARTFEED_BOOKMARKS_ERROR);
	}
	while ($row = $db->sql_fetchrow($result))
	{
		$bookmarked_topics[] = intval($row['topic_id']);
	}
	$db->sql_freeresult($result);
	if (count($bookmarked_topics) > 0)
	{
		$fetched_forums_str = ' AND ' . $db->sql_in_set('t.topic_id', $bookmarked_topics);
	}
	else
	{
		// Logically, if there are no bookmarked topics for this $user_id then there will be nothing in the feed.
		// Send a message to this effect in the feed.
		handle_error(SMARTFEED_NO_BOOKMARKS);
	}

}
else
{

	// Determining forum permissions is more complex under phpBB 3. The essential information is stored in the user_permissions column in the phpbb_users
	// table, typically in a nasty BLOB object. I am sure this is very efficient, but it is confusing as the dickens to us poor mod authors!
	
	// Begin code stolen and tweaked from auth.php
	
	$sql = 'SELECT auth_option, is_global, is_local
				FROM ' . ACL_OPTIONS_TABLE . '
				ORDER BY auth_option_id';
	if ( !($result = $db->sql_query($sql)) )
	{
		handle_error(SMARTFEED_ACL_OPTIONS_TABLE_ERROR);
	}
	
	$local = 0;
	$acl_options = array();
	$acl = array();
	while ($row = $db->sql_fetchrow($result))
	{
		if ($row['is_local'])
		{
			$acl_options['local'][$row['auth_option']] = $local++;
		}
	}
	$db->sql_freeresult($result);
	
	$user_permissions = explode("\n", $user_permissions);
	
	foreach ($user_permissions as $f => $seq)
	{
		if ($seq)
		{
			$i = 0;
	
			if (!isset($acl[$f]))
			{
				$acl[$f] = '';
			}
	
			while ($subseq = substr($seq, $i, 6))
			{
				// We put the original bitstring into the acl array
				$acl[$f] .= str_pad(base_convert($subseq, 36, 2), 31, 0, STR_PAD_LEFT);
				$i += 6;
			}
		}
	}
	
	// End code stolen from auth.php
	
	$f_read_local_index = -1;
	foreach ($acl_options['local'] as $key => $value)
	{
		if ($key == 'f_read')
		{
			$f_read_local_index = $value;
			break;
		}
	}
	
	if ($f_read_local_index == -1)
	{
		handle_error(SMARTFEED_F_READ_ERROR);
	}

	// Create the allowed forums array. These are the forums for which this user has read privileges. He can select
	// ONLY from these forums. If they request they can choose from a smaller subset of these forums.
	
	$allowed_forums = array();
	foreach ($acl as $key => $value)
	{
		if (substr($value, $f_read_local_index, 1) == '1')
		{
			$allowed_forums[] = $key;
		}
	}
	if (count($allowed_forums) == 0)
	{
		// If this user cannot retrieve ANY forums, this suggests that this board is tightly locked down to members only,
		// or every member must belong to a user group or have special forum permissions
		handle_error(SMARTFEED_NO_ACCESSIBLE_FORUMS);
	}
	
	// Get the requested forums. If none are listed, user wants all forums for which they have read access.
	$requested_forums = array();
	$query_string = ( !empty($_SERVER['QUERY_STRING']) ) ? $_SERVER['QUERY_STRING'] : '';
	$params = explode('&', $query_string);
	foreach ($params as $item)
	{
		if (substr($item,0,5) == 'forum')
		{
			$requested_forums[] = substr($item,6);
		}
	}
	
	// To capture global announcements when forums are specified, we have to add the pseudo-forum with a forum_id = 0.
	if (count($requested_forums) > 0)
	{
		$requested_forums[] = '0';
	}
	
	// Sort requested forums by forum_id and ensure there are no duplicates
	asort($requested_forums);
	$requested_forums = array_unique($requested_forums);
	
	// The forums that will be fetched is the array intersection of the requested and allowed forums. This prevents hacking
	// the URL to get feeds a user is not supposed to get. If no forums are specified on the URL field
	// then all forums that this user is authorized to access is assumed.
	
	$fetched_forums = (count($requested_forums) > 0) ? array_intersect($allowed_forums, $requested_forums): $allowed_forums;
	$fetched_forums_str = ' AND ' . $db->sql_in_set('p.forum_id', $fetched_forums);

}

// ------ Fourth, create the snippets of SQL used to return the set of posts ------ //

// Create the SQL stub for the sort order
switch($order_by)
{
	case 'user':
		$topic_asc_desc = ($user_topic_sortby_dir == 'd') ? 'DESC' : '';
		switch($user_topic_sortby_type)
		{
			case 'a':
				$order_by_sql = "t.topic_first_poster_name $topic_asc_desc, ";
				break;
			case 't':
				$order_by_sql = "t.topic_last_post_time $topic_asc_desc, ";
				break;
			case 'r':
				$order_by_sql = "t.topic_replies $topic_asc_desc, ";
				break;
			case 's':
				$order_by_sql = "t.topic_title $topic_asc_desc, " ; 
				break;
			case 'v':
				$order_by_sql = "t.topic_views $topic_asc_desc, ";
				break;
		}
		$post_asc_desc = ($user_post_sortby_dir == 'd') ? 'DESC' : '';
		switch($user_post_sortby_type)
		{
			case 'a':
				$order_by_sql .= "u.username_clean $post_asc_desc";
				break;
			case 't':
				$order_by_sql .= "p.post_time $post_asc_desc";
				break;
			case 's':
				$order_by_sql .= "p.post_subject $post_asc_desc" ; 
				break;
		}
		break;
	case 'standard':
		$order_by_sql = 'f.left_id, f.right_id, t.topic_last_post_time, p.post_time';
		break;
	case 'standard_desc':
		$order_by_sql = 'f.left_id, f.right_id, t.topic_last_post_time, p.post_time DESC';
		break;
	case 'postdate':
		$order_by_sql = 'p.post_time';
		break;
	case 'postdate_desc':
		$order_by_sql = 'p.post_time DESC';
		break;
}

// Create the new topics only SQL stub
$new_topics_sql = '';
$topics_posts_join_sql = 'p.topic_id = t.topic_id ';
if ($topics_only)
{
	$new_topics_sql = " AND t.topic_time > $date_limit ";
	// Check for first_post_only switch
	if ($first_post_only)
	{
		$topics_posts_join_sql = ' t.topic_first_post_id = p.post_id AND t.forum_id = f.forum_id';
	}
}

// Create SQL to remove your posts from the feed
$remove_mine_sql = '';
if ($remove_mine == '1')
{
	$remove_mine_sql = " AND p.poster_id <> $user_id ";
}

// Create SQL to remove your foes from the feed
$filter_foes_sql = '';
$foes = array();
if ($filter_foes == '1')
{

	// Fetch your foes
	$sql = 'SELECT zebra_id 
				FROM ' . ZEBRA_TABLE . "
				WHERE user_id = $user_id AND foe = 1";
	if ( !($result = $db->sql_query($sql)))
	{
		handle_error(SMARTFEED_FOES_ERROR);
	}
	while ($row = $db->sql_fetchrow($result))
	{
		$foes[] = $row['zebra_id'];
	}
	$db->sql_freeresult($result);

	if (count($foes) > 0)
	{
		$filter_foes_sql = ' AND ' . $db->sql_in_set('p.poster_id', $foes, true);
	}
	
}

// At last, construct the SQL to return the relevant posts
			
$sql_array = array(
	'SELECT'	=> 'f.*, t.*, p.*, u.*',

	'FROM'		=> array(
		POSTS_TABLE => 'p',
		USERS_TABLE => 'u',
		TOPICS_TABLE => 't'	),

	'WHERE'		=> "$topics_posts_join_sql
				AND p.poster_id = u.user_id
				$date_limit_sql
				$fetched_forums_str
				$new_topics_sql
				$remove_mine_sql
				$filter_foes_sql",

	'ORDER_BY'	=> $order_by_sql
);

$sql_array['LEFT_JOIN'] = array(
	array(
		'FROM'	=> array(FORUMS_TABLE => 'f', TOPICS_TABLE => 't'),
		'ON'	=> 't.forum_id = f.forum_id'
	)
);

$sql = $db->sql_build_query('SELECT', $sql_array);

// Respect any hardcoded limit on the maximum number of items that can be in a newsfeed. Board owners may 
// have to place a limit to avoid using too much CPU or memory. Please note that the way the $db->sql_query_limit
// function works, it is only applied at runtime.
$limit = SMARTFEED_MAX_ITEMS;

// If the count_limit parameter is present and valid, it should be respected. However, the limit should not
// exceed SMARTFEED_MAX_ITEMS, if this value is not zero
if ($count_limit > 0)
{
	if (SMARTFEED_MAX_ITEMS > 0)
	{
		if (SMARTFEED_MAX_ITEMS > $count_limit)
		{
			$limit = $count_limit;
		}
		else
		{
			$limit = SMARTFEED_MAX_ITEMS;
		}
	}
	else
	{
		$limit = $count_limit;
	}
}

// Now finally, let's fetch the actual data
$result = $db->sql_query_limit($sql, $limit); // Execute the SQL to retrieve the relevant posts
$rowset = $db->sql_fetchrowset($result); // Get all the posts as a set

if ($show_pms)
{

	// If there are any unread private messages, they are fetched separately and passed as a rowset to publish_feed.
	$pm_sql = 'SELECT *
					FROM ' . PRIVMSGS_TO_TABLE . ' pt, ' . PRIVMSGS_TABLE . ' pm, ' . USERS_TABLE . " u
					WHERE pt.msg_id = pm.msg_id
						AND pt.author_id = u.user_id
						AND pt.user_id = $user_id
						AND pm_unread = 1";
	$pm_result = $db->sql_query($pm_sql);
	$pm_rowset = $db->sql_fetchrowset($pm_result);

}

publish_feed($rowset, $pm_rowset); // Publish the set of rows as feed items
$db->sql_freeresult($result);
$db->sql_freeresult($pm_result);

// Reset the user's last visit date on the forum, if so requested
if ($lastvisit)
{
	$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_lastvisit = ' . time() . " 
				WHERE user_id = $user_id";
		
	if ( !($result = $db->sql_query($sql)) )
	{
		handle_error(SMARTFEED_RESET_ERROR);
	}
}

// Kill the session we consumed. We don't want to use session_kill() because it updates user_lastvisit,
// which we don't necessarily want to do since the user may want to visit the board later and see
// messages in the feed as unread.
$sql = 'DELETE FROM ' . SESSIONS_TABLE . "
	WHERE session_id = '" . $db->sql_escape($session->session_id) . "'
		AND session_user_id = " . (int) $session->data['user_id'];
$db->sql_query($sql);

if (($session['user_id'] != ANONYMOUS) && ($session->cookie_data['k']))
{
	$sql = 'DELETE FROM ' . SESSIONS_KEYS_TABLE . '
		WHERE user_id = ' . (int) $session->data['user_id'] . "
			AND key_id = '" . $db->sql_escape(md5($session->cookie_data['k'])) . "'";
	$db->sql_query($sql);
}

// All done!
exit;

function handle_error($error_code)
{

	// Publish the error as a newsfeed with a single item in it.
	
	global $user, $encrypted_pswd, $user_id, $board_url, $phpEx, $phpbb_root_path, $feed_type, $_SERVER, $config;
	if (!(isset($feed_type)))
	{
		$feed_type = SMARTFEED_RSS_20_VALUE;
	}
	
	switch($error_code)
	{
		case SMARTFEED_NO_ERRORS:
			// This should not happen, but just in case exit with no error
			return;
		case SMARTFEED_NO_ARGUMENTS:
			$error_msg = $user->lang['SMARTFEED_NO_ARGUMENTS'];
			break;
		case SMARTFEED_NO_U_ARGUMENT:
			$error_msg = $user->lang['SMARTFEED_NO_U_ARGUMENT'];
			break;
		case SMARTFEED_NO_E_ARGUMENT:
			$error_msg = $user->lang['SMARTFEED_NO_E_ARGUMENT'];
			break;
		case SMARTFEED_USER_TABLE_COUNT_ERROR:
			$error_msg = $user->lang['SMARTFEED_USER_TABLE_COUNT_ERROR'];
			break;
		case SMARTFEED_USER_ID_DOES_NOT_EXIST:
			$error_msg = $user->lang['SMARTFEED_USER_ID_DOES_NOT_EXIST'];
			break;
		case SMARTFEED_USER_TABLE_PASSWORD_ERROR:
			$error_msg = $user->lang['SMARTFEED_USER_TABLE_PASSWORD_ERROR'];
			break;
		case SMARTFEED_BAD_PASSWORD_ERROR:
			$error_msg = sprintf($user->lang['SMARTFEED_BAD_PASSWORD_ERROR'], $encrypted_pswd, $user_id, $board_url . "smartfeed_url.$phpEx");
			break;
		case SMARTFEED_IP_AUTH_ERROR:
			$error_msg = $user->lang['SMARTFEED_IP_AUTH_ERROR'];
			break;
		case SMARTFEED_LIMIT_FORMAT_ERROR:
			$error_msg = $user->lang['SMARTFEED_LIMIT_FORMAT_ERROR'];
			break;
		case SMARTFEED_FEED_TYPE_ERROR:
			$error_msg = $user->lang['SMARTFEED_FEED_TYPE_ERROR'];
			break;
		case SMARTFEED_SORT_BY_ERROR:
			$error_msg = $user->lang['SMARTFEED_SORT_BY_ERROR'];
			break;
		case SMARTFEED_TOPICS_ONLY_ERROR:
			$error_msg = $user->lang['SMARTFEED_TOPICS_ONLY_ERROR'];
			break;
		case SMARTFEED_FIRST_POST_ONLY_ERROR:
			$error_msg = $user->lang['SMARTFEED_FIRST_POST_ONLY_ERROR'];
			break;
		case SMARTFEED_REMOVE_MINE_ERROR:
			$error_msg = $user->lang['SMARTFEED_REMOVE_MINE_ERROR'];
			break;
		case SMARTFEED_MAX_WORD_SIZE_ERROR:
			$error_msg = $user->lang['SMARTFEED_MAX_WORD_SIZE_ERROR'];
			break;
		case SMARTFEED_BAD_PMS_VALUE:
			$error_msg = $user->lang['SMARTFEED_BAD_PMS_VALUE'];
			break;
		case SMARTFEED_COUNT_LIMIT_CONSISTENCY_ERROR:
			$error_msg = $user->lang['SMARTFEED_COUNT_LIMIT_CONSISTENCY_ERROR'];
			break;
		case SMARTFEED_COUNT_LIMIT_ERROR:
			$error_msg = $user->lang['SMARTFEED_COUNT_LIMIT_ERROR'];
			break;
		case SMARTFEED_FILTER_FOES_ERROR:
			$error_msg = $user->lang['SMARTFEED_FILTER_FOES_ERROR'];
			break;
		case SMARTFEED_BAD_BOOKMARKS_VALUE:
			$error_msg = $user->lang['SMARTFEED_BAD_BOOKMARKS_VALUE'];
			break;
		case SMARTFEED_BOARD_DISABLED:
			$error_msg = $user->lang['SMARTFEED_BOARD_DISABLED'];
			break;
		case SMARTFEED_F_READ_ERROR:
			$error_msg = $user->lang['SMARTFEED_F_READ_ERROR'];
			break;
		case SMARTFEED_ACL_OPTIONS_TABLE_ERROR:
			$error_msg = $user->lang['SMARTFEED_ACL_OPTIONS_TABLE_ERROR'];
			break;
		case SMARTFEED_USER_ERROR:
			$error_msg = $user->lang['SMARTFEED_USER_ERROR'];
			break;
		case SMARTFEED_FOES_ERROR:
			$error_msg = $user->lang['SMARTFEED_FOES_ERROR'];
			break;
		case SMARTFEED_BOOKMARKS_ERROR:
			$error_msg = $user->lang['SMARTFEED_BOOKMARKS_ERROR'];
			break;
		case SMARTFEED_NO_BOOKMARKS:
			$error_msg = $user->lang['SMARTFEED_NO_BOOKMARKS'];
			break;
		case SMARTFEED_NO_ACCESSIBLE_FORUMS:
			$error_msg = $user->lang['SMARTFEED_NO_ACCESSIBLE_FORUMS'];
			break;
		case SMARTFEED_LASTVISIT_ERROR:
			$error_msg = $user->lang['SMARTFEED_LASTVISIT_ERROR'];
			break;
		case SMARTFEED_RESET_ERROR:
			$error_msg = $user->lang['SMARTFEED_RESET_ERROR'];
			break;
		case SMARTFEED_MIN_WORD_SIZE_ERROR:
			$error_msg = $user->lang['SMARTFEED_MIN_WORD_SIZE_ERROR'];
			break;
		case SMARTFEED_STYLE_ERROR:
			$error_msg = $user->lang['SMARTFEED_STYLE_ERROR'];
			break;
		case SMARTFEED_BAD_AUTH_METHOD:
			$error_msg = $user->lang['SMARTFEED_BAD_AUTH_METHOD'];
			break;
		case SMARTFEED_APACHE_AUTHENTICATION_WARNING_REG:
			$error_msg = $user->lang['SMARTFEED_APACHE_AUTHENTICATION_WARNING_REG'];
			break;
		case SMARTFEED_PM_UPDATE_ERROR:
			$error_msg = $user->lang['SMARTFEED_PM_UPDATE_ERROR'];
			break;
		default:
			$error_msg = sprintf($user->lang['SMARTFEED_UNKNOWN_ERROR'], $error_code);
			break;
	}
	publish_feed($error_msg);
	
	exit;

}

function publish_feed($rowset, $pm_rowset = NULL)
{

	// This function does the grunt work of changing rows in a SQL query into a newsfeed. It also handles
	// any potential errors that may have occurred by placing the error as the sole item of the feed itself. 
	
	global $board_url, $user, $feed_type, $phpEx, $config, $phpbb_root_path, $db, $registered_user, $max_word_size, $min_word_size, $feed_style, $user_id;
   	
	require($phpbb_root_path . 'includes/feedcreator.class.' . $phpEx); 

	// Override the Feed Creator Generator information to include info on this modification
	define('TIME_ZONE', str_replace('.',':',sprintf("%+'06.2f",$config['board_timezone'])));

	if (is_string($rowset)) // If the first argument is a string rather than an array, some error occurred so we need to publish the error as a newsfeed item
	{
		$error_mode = true;
		$error_msg = &$rowset;	// The argument contains the error message
	}
	else if (is_array($rowset))
	{
		// This is the typical case. We are publishing a newsfeed
		$error_mode = false;
	}
	else
	{
		// This case should never happen but if it does, the programmer must have messed up.
		$error_mode = true;
		$error_msg =  $user->lang['SMARTFEED_PUBLISH_FEED_ARGUMENT_ERROR'];
	}
		
	// Initialize the feed
	$rss = new UniversalFeedCreator();
	$rss->useCached();
	$rss->title = $user->lang['SMARTFEED_FEED_TITLE'];
	$rss->description = $user->lang['SMARTFEED_FEED_DESCRIPTION'];
	$rss->link = ($feed_type == SMARTFEED_ATOM_10_VALUE) ? $board_url . "smartfeed.$phpEx?" . $_SERVER['QUERY_STRING'] : $board_url . "smartfeed.$phpEx?" . htmlspecialchars($_SERVER['QUERY_STRING']);
	$rss->syndicationURL = $board_url . "smartfeed.$phpEx";
	
	// Some RSS 2.0 tags
	$rss->language = SMARTFEED_RFC1766_LANG;
	$rss->pubDate = time();
	$rss->ttl = SMARTFEED_TTL;
	$rss->copyright = $user->lang['SMARTFEED_COPYRIGHT'];
	$rss->editor = SMARTFEED_EDITOR;
	$rss->editorEmail = SMARTFEED_EDITOR_EMAIL;
	$rss->webmaster = SMARTFEED_WEBMASTER;
	
	// Put an image in the feed. Typically this will be the phpBB default image
	$image = new FeedImage();
	$image->title = $user->lang['SMARTFEED_FEED_TITLE']; // Enhance to use default logo if not specified
	$image->url = $board_url . 'styles/' . get_default_style() . '/' . SMARTFEED_FEED_IMAGE_PATH; 
	$image->link = $rss->link;
	$image->description = strip_tags($config['site_desc']);
	$rss->image = $image;
	
	if ($error_mode)
	{
		// Place the error as a feed item
		$item = new FeedItem();
		$item->title = $user->lang['SMARTFEED_ERROR_TITLE'];
		$item->description = sprintf($user->lang['SMARTFEED_ERROR_INTRODUCTION'], $board_url . "smartfeed_url.$phpEx") . $error_msg;
		$item->author = $config['board_contact'];
		$item->authorEmail = $config['board_email'];
		$rss->addItem($item);
	}
	else
	{
	
		// If there are any unread private messages, publish them first
		if (is_array($pm_rowset) && (count($pm_rowset) > 0))
		{
		
			foreach ($pm_rowset as $row)
			{

				// This private message goes in the newsfeed
				$user_timezone = (float) $row['user_timezone'];
				$username = $row['username']; // Don't need to worry about Anonymous users for private messages, they cannot send them
	
				$item = new FeedItem();
	
				// Create the title for the private message
				$title = $user->lang['PRIVATE_MESSAGE'] . $user->lang['SMARTFEED_DELIMITER'] . $row['message_subject'] . $user->lang['SMARTFEED_DELIMITER'] . $user->lang['FROM'] . ' ' . $username;
				
				$item->title = censor_text($title);
				
				// Create a number of item attributes, not all of which are necessarily in the feed (it depends on the feed type)
				$item->date = $row['message_time'] + ($user_timezone * 3600);
				$item->pubDate = $row['message_time'] + ($user_timezone * 3600);
				$item->link = $board_url . 'ucp.' . $phpEx . '?i=pm&mode=view&f=0&p=' . $row['msg_id'];
				$item->source = $board_url;
				$item->category = $user->lang['PRIVATE_MESSAGE'];
				$item->guid = $item->link;
				$item->descriptionHtmlSyndicated = true;
				// The comment URL is not generated until the message is read, so we have to use the same URL as the view message link
				$item->comments = $board_url . 'ucp.' . $phpEx . '?i=pm&mode=view&f=0&p=' . $row['msg_id'];
				$item->author = $username;
	
				if ($feed_type==SMARTFEED_RSS_20_VALUE) 
				{
					// RSS 2.0 requires an email field to validate. Use a fake email address (no_email@example.com), set in feed_creator.class.php 
					// if the author field is blank) unless in user profile it says it is okay to show email address.
					$item->authorEmail = (($row['user_allow_viewemail'] == '1') && (SMARTFEED_PRIVACY_MODE == false)) ? $row['user_email'] : '';
				}               
				else
				{
					$item->authorEmail = '';
				}
				
				// To "dress up" the message text with bbCode, images, smilies etc., we need to use generate_text_for_display() function. This is a great
				// function. Replicating this for phpBB 2 was a pain in the buttocks!
				
				$message = censor_text($row['message_text']);
				
				$user_sig = ( $row['enable_sig'] && ($row['user_sig'] != '') && $config['allow_sig'] && (SMARTFEED_PRIVACY_MODE == false) ) ? $row['user_sig'] : '';
				
				if ($feed_style == 'HTML')
				{
					$flags = (($row['enable_bbcode']) ? OPTION_FLAG_BBCODE : 0) +
						(($row['enable_smilies']) ? OPTION_FLAG_SMILIES : 0) + 
						(($row['enable_magic_url']) ? OPTION_FLAG_LINKS : 0);
						
					$message = generate_text_for_display($message, $row['bbcode_uid'], $row['bbcode_bitfield'], $flags);
		
					if ($user_sig != '')
					{
						$user_sig = generate_text_for_display(censor_text($user_sig), $row['user_sig_bbcode_uid'], $row['user_sig_bbcode_bitfield'], $flags);
					}
		
					$message = ($user_sig <> '') ? $message . SMARTFEED_POST_SIGNATURE_DELIMITER . $user_sig : $message;
					
				}
				else
				{
					// Either Compact or Basic Style wanted
					if ($feed_style == 'BASIC')
					{
						$message = ($user_sig <> '') ? $message . "\n\n" . $user_sig : $message;
					}
					$message = strip_tags($message);	// Gets rid of any embedded HTML
					strip_bbcode(&$message); 			// Remove the BBCode
					// Either condense all text or make line feeds explicit
					$message = ($feed_style == 'BASIC') ? str_replace("\n", '<br />', $message) : str_replace("\n", ' ', $message);
				}
				
				// Add the item (post) to the feed
				$item->description = $message;
				$rss->addItem($item);
				
				// Mark the private message as read
				$sql = 'UPDATE ' . PRIVMSGS_TO_TABLE . "
						SET pm_unread = 0 
						WHERE msg_id = " . $row['msg_id'] . " 
							AND user_id = $user_id
							AND author_id = " . $row['author_id'] . " 
							AND folder_id = " . $row['folder_id'];
				if ( !($result = $db->sql_query($sql)) )
				{
					handle_error(SMARTFEED_PM_UPDATE_ERROR);
				}
			}
			
		}
		
		// loop through the rowset, each row is a item in the feed
		foreach ($rowset as $row)
		{
		
			// Handle the logic to ignore a post if it does not contain the minimum number of words needed
			if (!( ($min_word_size != '') && truncate_words($row['post_text'], intval($max_word_size), true) < $min_word_size))
			{
			
				// This post goes in the newsfeed

				$user_timezone = (float) $row['user_timezone'];
				$username = ($row['user_id'] == ANONYMOUS) ? $row['post_username'] : $row['username'];
	
				$item = new FeedItem();
	
				// Create the title for the item (post)
				$forum_name = ($row['forum_name'] == NULL) ? $user->lang['SMARTFEED_GLOBAL_ANNOUNCEMENT'] : $row['forum_name'];
				$title = (SMARTFEED_SUPPRESS_FORUM_NAMES) ? $row['post_subject'] : $forum_name . $user->lang['SMARTFEED_DELIMITER'] . $row['post_subject'];
				if ($row['topic_first_post_id'] <> $row['post_id'])
				{
					$first_post_for_topic = false;
					if (SMARTFEED_SHOW_USERNAME_IN_REPLIES)
					{
						$title .= ($row['username'] == '') ? $user->lang['SMARTFEED_DELIMITER'] . $user->lang['SMARTFEED_REPLY_BY'] . ' ' . $user->lang['GUEST'] : $user->lang['SMARTFEED_DELIMITER'] . $user->lang['SMARTFEED_REPLY_BY'] . ' ' . $username;
					}
					else
					{
						$title .= $user->lang['SMARTFEED_DELIMITER'] . $user->lang['SMARTFEED_REPLY'];
					}
				}
				else
				{
					$first_post_for_topic = true;
					if (SMARTFEED_SHOW_USERNAME_IN_FIRST_TOPIC_POST)
					{
						$title .= $user->lang['SMARTFEED_DELIMITER'] . $user->lang['AUTHOR'] . ' ' . $username;
					}
				}
				
				$item->title = censor_text($title);
				
				// Create a number of item attributes, not all of which are necessarily in the feed (it depends on the feed type)
				$item->date = $row['post_time'] + ($user_timezone * 3600);
				$item->pubDate = $row['post_time'] + ($user_timezone * 3600);
				$item->link = $board_url . 'viewtopic.' . $phpEx . '?f=' . $row['forum_id'] . '&t=' . $row['topic_id'] . '#p' . $row['post_id'];
				$item->source = $board_url;
				$item->category = $row['forum_name'];
				$item->guid = $item->link;
				$item->descriptionHtmlSyndicated = true;
				$item->comments = $board_url . 'posting.' . $phpEx . '?mode=reply&f=' . $row['forum_id'] . '&t=' . $row['topic_id'];
				$item->author = $username;
	
				if ($feed_type==SMARTFEED_RSS_20_VALUE) 
				{
					// RSS 2.0 requires an email field to validate. Use a fake email address (no_email@example.com, set in feed_creator.class.php 
					// if the author field is blank) unless in user profile it says it is okay to show email address.
					$item->authorEmail = (($row['user_allow_viewemail'] == '1') && (SMARTFEED_PRIVACY_MODE == false)) ? $row['user_email'] : '';
				}               
				else
				{
					$item->authorEmail = '';
				}
				
				// To "dress up" the post text with bbCode, images, smilies etc., we need to use generate_text_for_display() function. This is a great
				// function. Replicating this for phpBB 2 was a pain in the buttocks!
				
				$message = censor_text($row['post_text']);
				
				$user_sig = ( $row['enable_sig'] && $row['user_sig'] != '' && $config['allow_sig'] && ((SMARTFEED_PRIVACY_MODE == false) || $registered_user) ) ? $row['user_sig'] : '';
				
				if ($feed_style == 'HTML')
				{
					$flags = (($row['enable_bbcode']) ? OPTION_FLAG_BBCODE : 0) +
						(($row['enable_smilies']) ? OPTION_FLAG_SMILIES : 0) + 
						(($row['enable_magic_url']) ? OPTION_FLAG_LINKS : 0);
						
					$message = generate_text_for_display($message, $row['bbcode_uid'], $row['bbcode_bitfield'], $flags);
		
					if ($user_sig != '')
					{
						$user_sig = generate_text_for_display(censor_text($user_sig), $row['user_sig_bbcode_uid'], $row['user_sig_bbcode_bitfield'], $flags);
					}
		
					$message = ($user_sig <> '') ? $message . SMARTFEED_POST_SIGNATURE_DELIMITER . $user_sig : $message;
					
				}
				else
				{
					// Either Compact or Basic Style wanted
					if ($feed_style == 'BASIC')
					{
						$message = ($user_sig <> '') ? $message . "\n\n" . $user_sig : $message;
					}
					$message = strip_tags($message);	// Gets rid of any embedded HTML
					strip_bbcode(&$message); 			// Remove the BBCode
					// Either condense all text or make line feeds explicit
					$message = ($feed_style == 'BASIC') ? str_replace("\n", '<br />', $message) : str_replace("\n", ' ', $message);
				}
				
				// Handle the maximum number of words requested per post logic
				if ($max_word_size != $user->lang['SMARTFEED_MAX_WORDS_WANTED'])
				{
					$message = truncate_words($message, intval($max_word_size));
				}
				
				// Add the item (post) to the feed
				$item->description = $message;
				$rss->addItem($item);
				
			}
			
		}
	}
	
	// Publish the feed as RSS or Atom
	$rss->outputFeed($feed_type);
	return;

}

function decrypt($encoded_64, $key)
{

	// This function decrypts $encoded_64 with the given $key. It uses the PHP Secure Communications Library, found
	// at http://phpseclib.sourceforge.net/. If mcrypt is compiled into PHP it will use that, otherwise it will use the functions 
	// from this library. This allows for maximum portability of Smartfeed. Thanks to Terrafrost at phpbb.com for the suggestion
	// to use this library!

	global $phpbb_root_path;
	
	include($phpbb_root_path . 'includes/Crypt/TripleDES.php');
	$des = new Crypt_TripleDES();

    $des->setKey($key);

	// Decrypt
	$decrypted_data = $des->decrypt(base64_decode($encoded_64));
	return $decrypted_data;
	
}  

function get_default_style()
{

	// This function returns the default style. The default style is needed to retrive the feed
	// image.

	global $config, $db;
	
	$sql = 'SELECT template_path 
			FROM ' . STYLES_TABLE . ' s JOIN ' . STYLES_TEMPLATE_TABLE . ' t ON s.template_id = t.template_id 
			WHERE style_id = ' . $config[default_style];
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	return $row['template_path'];
	
}  

function truncate_words($text, $max_words, $just_count_words = false)
{

	// This function returns the first $max_words from the supplied $text. If $just_count_words == true, a word count is returned
	
	global $user;
	
	$word_array = explode(' ',$text);
	if ($just_count_words)
	{
		return count($word_array);
	}
	
	if (count($word_array) < $max_words)
	{
		return rtrim($text);
	}
	else
	{
		$truncated_text = '';
		for ($i=0; $i < $max_words; $i++) 
		{
			$truncated_text .= $word_array[$i] . ' ';
		}
		return rtrim($truncated_text) . $user->lang['SMARTFEED_MAX_WORDS_NOTIFIER'];
	}
}

?>
